@@ -29,6 +29,8 @@ var dataset
 
 var currentQuestion
 
+var hasWon
+
 function start(items_, dataset_, mode_) {
     if (mode_ == "Colors")
         Core.checkForVoices(items_.background);
@@ -59,6 +61,7 @@ function initLevel() {
 
     // Shuffle again not to ask the question in the model order
     dataset[currentLevel] = Core.shuffle(dataset[currentLevel])
+    hasWon = false
     initQuestion()
 }
 
@@ -73,6 +76,7 @@ function initQuestion() {
 function nextQuestion() {
     if(dataset[currentLevel].length <= currentQuestion + 1) {
         items.bonus.good("flower")
+        hasWon = true
     } else {
         currentQuestion++
         items.score.currentSubLevel++
