@@ -51,7 +51,8 @@ function start() {
     // register the voices for the locale
     GCompris.DownloadManager.updateResource(GCompris.DownloadManager.getVoicesResourceForLocale(locale))
 
-    dataset = Lang.load(items.parser, langUrl, "words.json",
+    dataset = Lang.load(items.parser, langUrl,
+                        GCompris.ApplicationSettings.wordset ? "words.json" : "words_sample.json",
                         "content-"+ locale +".json")
 
     // If dataset is empty, we try to load from short locale
@@ -65,7 +66,8 @@ function start() {
         } else {
             localeShort = locale;
         }
-        dataset = Lang.load(items.parser, langUrl, "words.json",
+        dataset = Lang.load(items.parser, langUrl,
+                            GCompris.ApplicationSettings.wordset ? "words.json" : "words_sample.json",
                             "content-"+localeShort+ ".json")
     }
 
@@ -73,7 +75,9 @@ function start() {
     if(!dataset) {
         // English fallback
         items.background.englishFallback = true
-        dataset = Lang.load(items.parser, langUrl, "words.json", "content-en.json")
+        dataset = Lang.load(items.parser, langUrl,
+                            GCompris.ApplicationSettings.wordset ? "words.json" : "words_sample.json",
+                            "content-en.json")
     } else {
         items.background.englishFallback = false
     }
@@ -164,7 +168,7 @@ function getRandomMaskedQuestion(clearQuestion, guessLetters, level) {
 }
 
 function sortUnique(arr) {
-    arr = arr.sort(function (a, b) { return a.localeCompare(b); });
+    arr = GCompris.ApplicationInfo.localeSort(arr, items.locale);
     var ret = [arr[0]];
     for (var i = 1; i < arr.length; i++) { // start loop at 1 as element 0 can never be a duplicate
         if (arr[i-1] !== arr[i]) {
@@ -194,7 +198,7 @@ function showQuestion() {
     items.answer = question.answer
     items.answers.model = question.choices
     items.questionText.text = question.maskedQuestion
-    items.questionImage.source = "qrc:/gcompris/data/" + question.image
+    items.questionImage.source = question.image
 }
 
 function nextLevel() {
