@@ -1,6 +1,6 @@
-/* GCompris - ApplicationSettingsDefault.cpp
+/* GCompris - ApplicationSettings.cpp
  *
- * Copyright (C) 2014 Johnny Jazeix <jazeix@gmail.com>
+ * Copyright (C) 2014-2016 Johnny Jazeix <jazeix@gmail.com>
  *
  * Authors:
  *   Johnny Jazeix <jazeix@gmail.com>
@@ -22,12 +22,17 @@
 #include "ApplicationSettings.h"
 #include "ApplicationInfo.h"
 
+#include "DownloadManager.h"
+
 #include <qmath.h>
 #include <QUrl>
 #include <QUrlQuery>
 #include <QGuiApplication>
 #include <QScreen>
 #include <QLocale>
+#include <QApplication>
+#include <QRect>
+#include <QDesktopWidget>
 
 #include <QSettings>
 #include <QStandardPaths>
@@ -37,6 +42,7 @@
 
 #define GC_DEFAULT_FONT "Andika-R.ttf"
 #define GC_DEFAULT_FONT_CAPITALIZATION 0 // Font.MixedCase
+#define GC_DEFAULT_FONT_LETTER_SPACING 0
 
 static const QString GENERAL_GROUP_KEY = "General";
 static const QString ADMIN_GROUP_KEY = "Admin";
@@ -44,6 +50,8 @@ static const QString INTERNAL_GROUP_KEY = "Internal";
 static const QString FAVORITE_GROUP_KEY = "Favorite";
 
 static const QString FULLSCREEN_KEY = "fullscreen";
+static const QString PREVIOUS_HEIGHT_KEY = "previousHeight";
+static const QString PREVIOUS_WIDTH_KEY = "previousWidth";
 static const QString SHOW_LOCKED_ACTIVITIES_KEY = "showLockedActivities";
 static const QString ENABLE_AUDIO_VOICES_KEY = "enableAudioVoices";
 static const QString ENABLE_AUDIO_EFFECTS_KEY = "enableAudioEffects";
@@ -56,18 +64,22 @@ static const QString ENABLE_AUTOMATIC_DOWNLOADS = "enableAutomaticDownloads";
 static const QString DOWNLOAD_SERVER_URL_KEY = "downloadServerUrl";
 
 static const QString EXE_COUNT_KEY = "exeCount";
+static const QString LAST_GC_VERSION_RAN = "lastGCVersionRan";
 
 static const QString FILTER_LEVEL_MIN = "filterLevelMin";
 static const QString FILTER_LEVEL_MAX = "filterLevelMax";
 
 static const QString BASE_FONT_SIZE_KEY = "baseFontSize";
 static const QString FONT_CAPITALIZATION = "fontCapitalization";
+static const QString FONT_LETTER_SPACING = "fontLetterSpacing";
 
 static const QString DEFAULT_CURSOR = "defaultCursor";
 static const QString NO_CURSOR = "noCursor";
 static const QString DEMO_KEY = "demo";
+static const QString CODE_KEY = "key";
 static const QString KIOSK_KEY = "kiosk";
 static const QString SECTION_VISIBLE = "sectionVisible";
+static const QString WORDSET = "wordset";
 
 static const QString PROGRESS_KEY = "progress";
 
@@ -75,29 +87,45 @@ ApplicationSettings *ApplicationSettings::m_instance = NULL;
 
 ApplicationSettings::ApplicationSettings(QObject *parent): QObject(parent),
      m_baseFontSizeMin(-7), m_baseFontSizeMax(7),
+     m_fontLetterSpacingMin(0.0), m_fontLetterSpacingMax(8.0),
      m_config(QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) +
               "/gcompris/" + GCOMPRIS_APPLICATION_NAME + ".conf", QSettings::IniFormat)
 {
+    const QRect &screenSize = QApplication::desktop()->screenGeometry();
+
     // initialize from settings file or default
 
     // general group
     m_config.beginGroup(GENERAL_GROUP_KEY);
     m_isAudioEffectsEnabled = m_config.value(ENABLE_AUDIO_EFFECTS_KEY, true).toBool();
     m_isFullscreen = m_config.value(FULLSCREEN_KEY, true).toBool();
-	m_isAudioVoicesEnabled = m_config.value(ENABLE_AUDIO_VOICES_KEY, true).toBool();
+    m_previousHeight = m_config.value(PREVIOUS_HEIGHT_KEY, screenSize.height()).toUInt();
+    m_previousWidth = m_config.value(PREVIOUS_WIDTH_KEY, screenSize.width()).toUInt();
+    m_isAudioVoicesEnabled = m_config.value(ENABLE_AUDIO_VOICES_KEY, true).toBool();
     m_isVirtualKeyboard = m_config.value(VIRTUALKEYBOARD_KEY,
             ApplicationInfo::getInstance()->isMobile()).toBool();
     m_locale = m_config.value(LOCALE_KEY, GC_DEFAULT_LOCALE).toString();
     m_font = m_config.value(FONT_KEY, GC_DEFAULT_FONT).toString();
     m_fontCapitalization = m_config.value(FONT_CAPITALIZATION, GC_DEFAULT_FONT_CAPITALIZATION).toUInt();
+    setFontLetterSpacing(m_config.value(FONT_LETTER_SPACING, GC_DEFAULT_FONT_LETTER_SPACING).toReal());
     m_isEmbeddedFont = m_config.value(IS_CURRENT_FONT_EMBEDDED, true).toBool();
 
-// The default demo mode based on the platform
-#if defined(WITH_ACTIVATION_CODE)
-    m_isDemoMode = m_config.value(DEMO_KEY, true).toBool();
-#else
-    m_isDemoMode = m_config.value(DEMO_KEY, false).toBool();
-#endif
+    // Init the activation mode
+    if(QLatin1String(ACTIVATION_MODE) == "no")
+        m_activationMode = 0;
+    else if(QLatin1String(ACTIVATION_MODE) == "inapp")
+        m_activationMode = 1;
+    else if(QLatin1String(ACTIVATION_MODE) == "internal")
+        m_activationMode = 2;
+    else
+        qFatal("Unknown activation mode");
+
+    // Set the demo mode
+    if(QLatin1String(ACTIVATION_MODE) != "no")
+        m_isDemoMode = m_config.value(DEMO_KEY, true).toBool();
+    else
+        m_isDemoMode = false;
+    m_codeKey = m_config.value(CODE_KEY, "").toString();
 
 #if defined(WITH_KIOSK_MODE)
     m_isKioskMode = m_config.value(KIOSK_KEY, true).toBool();
@@ -108,16 +136,17 @@ ApplicationSettings::ApplicationSettings(QObject *parent): QObject(parent),
     // Option only useful if we are in demo mode (else all the activities are available and unlocked)
     // By default, all the activities are displayed (even locked ones)
     m_showLockedActivities = m_config.value(SHOW_LOCKED_ACTIVITIES_KEY, m_isDemoMode).toBool();
-	m_sectionVisible = m_config.value(SECTION_VISIBLE, true).toBool();
-	m_isAutomaticDownloadsEnabled = m_config.value(ENABLE_AUTOMATIC_DOWNLOADS,
+    m_sectionVisible = m_config.value(SECTION_VISIBLE, true).toBool();
+    m_wordset = m_config.value(WORDSET, "").toString();
+    m_isAutomaticDownloadsEnabled = m_config.value(ENABLE_AUTOMATIC_DOWNLOADS,
             !ApplicationInfo::getInstance()->isMobile() && ApplicationInfo::isDownloadAllowed()).toBool();
     m_filterLevelMin = m_config.value(FILTER_LEVEL_MIN, 1).toUInt();
     m_filterLevelMax = m_config.value(FILTER_LEVEL_MAX, 6).toUInt();
-	m_defaultCursor = m_config.value(DEFAULT_CURSOR, false).toBool();
-	m_noCursor = m_config.value(NO_CURSOR, false).toBool();
+    m_defaultCursor = m_config.value(DEFAULT_CURSOR, false).toBool();
+    m_noCursor = m_config.value(NO_CURSOR, false).toBool();
     setBaseFontSize(m_config.value(BASE_FONT_SIZE_KEY, 0).toInt());
 
-	m_config.sync();  // make sure all defaults are written back
+    m_config.sync();  // make sure all defaults are written back
     m_config.endGroup();
 
     // admin group
@@ -128,6 +157,7 @@ ApplicationSettings::ApplicationSettings(QObject *parent): QObject(parent),
     // internal group
     m_config.beginGroup(INTERNAL_GROUP_KEY);
     m_exeCount = m_config.value(EXE_COUNT_KEY, 0).toUInt();
+    m_lastGCVersionRan = m_config.value(LAST_GC_VERSION_RAN, 0).toUInt();
     m_config.endGroup();
 
     // no group
@@ -137,18 +167,22 @@ ApplicationSettings::ApplicationSettings(QObject *parent): QObject(parent),
 	connect(this, &ApplicationSettings::audioVoicesEnabledChanged, this, &ApplicationSettings::notifyAudioVoicesEnabledChanged);
 	connect(this, &ApplicationSettings::audioEffectsEnabledChanged, this, &ApplicationSettings::notifyAudioEffectsEnabledChanged);
 	connect(this, &ApplicationSettings::fullscreenChanged, this, &ApplicationSettings::notifyFullscreenChanged);
+    connect(this, &ApplicationSettings::previousHeightChanged, this, &ApplicationSettings::notifyPreviousHeightChanged);
+    connect(this, &ApplicationSettings::previousWidthChanged, this, &ApplicationSettings::notifyPreviousWidthChanged);
     connect(this, &ApplicationSettings::localeChanged, this, &ApplicationSettings::notifyLocaleChanged);
     connect(this, &ApplicationSettings::fontChanged, this, &ApplicationSettings::notifyFontChanged);
     connect(this, &ApplicationSettings::virtualKeyboardChanged, this, &ApplicationSettings::notifyVirtualKeyboardChanged);
     connect(this, &ApplicationSettings::automaticDownloadsEnabledChanged, this, &ApplicationSettings::notifyAutomaticDownloadsEnabledChanged);
     connect(this, &ApplicationSettings::filterLevelMinChanged, this, &ApplicationSettings::notifyFilterLevelMinChanged);
     connect(this, &ApplicationSettings::filterLevelMaxChanged, this, &ApplicationSettings::notifyFilterLevelMaxChanged);
-	connect(this, &ApplicationSettings::sectionVisibleChanged, this, &ApplicationSettings::notifySectionVisibleChanged);
+    connect(this, &ApplicationSettings::sectionVisibleChanged, this, &ApplicationSettings::notifySectionVisibleChanged);
+    connect(this, &ApplicationSettings::wordsetChanged, this, &ApplicationSettings::notifyWordsetChanged);
     connect(this, &ApplicationSettings::demoModeChanged, this, &ApplicationSettings::notifyDemoModeChanged);
     connect(this, &ApplicationSettings::kioskModeChanged, this, &ApplicationSettings::notifyKioskModeChanged);
     connect(this, &ApplicationSettings::downloadServerUrlChanged, this, &ApplicationSettings::notifyDownloadServerUrlChanged);
     connect(this, &ApplicationSettings::exeCountChanged, this, &ApplicationSettings::notifyExeCountChanged);
     connect(this, &ApplicationSettings::barHiddenChanged, this, &ApplicationSettings::notifyBarHiddenChanged);
+    connect(this, &ApplicationSettings::lastGCVersionRanChanged, this, &ApplicationSettings::notifyLastGCVersionRanChanged);
 }
 
 ApplicationSettings::~ApplicationSettings()
@@ -157,22 +191,27 @@ ApplicationSettings::~ApplicationSettings()
     // general group
     m_config.beginGroup(GENERAL_GROUP_KEY);
     m_config.setValue(SHOW_LOCKED_ACTIVITIES_KEY, m_showLockedActivities);
-	m_config.setValue(ENABLE_AUDIO_VOICES_KEY, m_isAudioVoicesEnabled);
+    m_config.setValue(ENABLE_AUDIO_VOICES_KEY, m_isAudioVoicesEnabled);
     m_config.setValue(LOCALE_KEY, m_locale);
     m_config.setValue(FONT_KEY, m_font);
     m_config.setValue(IS_CURRENT_FONT_EMBEDDED, m_isEmbeddedFont);
     m_config.setValue(FULLSCREEN_KEY, m_isFullscreen);
+    m_config.setValue(PREVIOUS_HEIGHT_KEY, m_previousHeight);
+    m_config.setValue(PREVIOUS_WIDTH_KEY, m_previousWidth);
     m_config.setValue(VIRTUALKEYBOARD_KEY, m_isVirtualKeyboard);
     m_config.setValue(ENABLE_AUTOMATIC_DOWNLOADS, m_isAutomaticDownloadsEnabled);
     m_config.setValue(FILTER_LEVEL_MIN, m_filterLevelMin);
-	m_config.setValue(FILTER_LEVEL_MAX, m_filterLevelMax);
+    m_config.setValue(FILTER_LEVEL_MAX, m_filterLevelMax);
     m_config.setValue(DEMO_KEY, m_isDemoMode);
+    m_config.setValue(CODE_KEY, m_codeKey);
     m_config.setValue(KIOSK_KEY, m_isKioskMode);
     m_config.setValue(SECTION_VISIBLE, m_sectionVisible);
-	m_config.setValue(DEFAULT_CURSOR, m_defaultCursor);
-	m_config.setValue(NO_CURSOR, m_noCursor);
+    m_config.setValue(WORDSET, m_wordset);
+    m_config.setValue(DEFAULT_CURSOR, m_defaultCursor);
+    m_config.setValue(NO_CURSOR, m_noCursor);
     m_config.setValue(BASE_FONT_SIZE_KEY, m_baseFontSize);
     m_config.setValue(FONT_CAPITALIZATION, m_fontCapitalization);
+    m_config.setValue(FONT_LETTER_SPACING, m_fontLetterSpacing);
     m_config.endGroup();
 
     // admin group
@@ -183,6 +222,7 @@ ApplicationSettings::~ApplicationSettings()
     // internal group
     m_config.beginGroup(INTERNAL_GROUP_KEY);
     m_config.setValue(EXE_COUNT_KEY, m_exeCount);
+    m_config.setValue(LAST_GC_VERSION_RAN, m_lastGCVersionRan);
     m_config.endGroup();
 
     m_config.sync();
@@ -232,12 +272,30 @@ void ApplicationSettings::notifyFontCapitalizationChanged()
     qDebug() << "new fontCapitalization: " << m_fontCapitalization;
 }
 
+void ApplicationSettings::notifyFontLetterSpacingChanged()
+{
+    updateValueInConfig(GENERAL_GROUP_KEY, FONT_LETTER_SPACING, m_fontLetterSpacing);
+    qDebug() << "new fontLetterSpacing: " << m_fontLetterSpacing;
+}
+
 void ApplicationSettings::notifyFullscreenChanged()
 {
     updateValueInConfig(GENERAL_GROUP_KEY, FULLSCREEN_KEY, m_isFullscreen);
     qDebug() << "fullscreen set to: " << m_isFullscreen;
 }
 
+void ApplicationSettings::notifyPreviousHeightChanged()
+{
+    updateValueInConfig(GENERAL_GROUP_KEY, PREVIOUS_HEIGHT_KEY, m_previousHeight);
+    qDebug() << "previous height set to: " << m_previousHeight;
+}
+
+void ApplicationSettings::notifyPreviousWidthChanged()
+{
+    updateValueInConfig(GENERAL_GROUP_KEY, PREVIOUS_WIDTH_KEY, m_previousWidth);
+    qDebug() << "previous width set to: " << m_previousWidth;
+}
+
 void ApplicationSettings::notifyVirtualKeyboardChanged()
 {
     updateValueInConfig(GENERAL_GROUP_KEY, VIRTUALKEYBOARD_KEY, m_isVirtualKeyboard);
@@ -278,6 +336,14 @@ void ApplicationSettings::notifyDemoModeChanged()
     qDebug() << "notifyDemoMode: " << m_isDemoMode;
 }
 
+void ApplicationSettings::notifyCodeKeyChanged()
+{
+    checkPayment();
+    if(!m_isDemoMode)
+        updateValueInConfig(GENERAL_GROUP_KEY, CODE_KEY, m_codeKey);
+    qDebug() << "notifyCodeKey: " << m_codeKey;
+}
+
 void ApplicationSettings::notifyKioskModeChanged()
 {
     updateValueInConfig(GENERAL_GROUP_KEY, KIOSK_KEY, m_isKioskMode);
@@ -286,8 +352,22 @@ void ApplicationSettings::notifyKioskModeChanged()
 
 void ApplicationSettings::notifySectionVisibleChanged()
 {
-	updateValueInConfig(GENERAL_GROUP_KEY, SECTION_VISIBLE, m_sectionVisible);
-	qDebug() << "notifySectionVisible: " << m_sectionVisible;
+    updateValueInConfig(GENERAL_GROUP_KEY, SECTION_VISIBLE, m_sectionVisible);
+    qDebug() << "notifySectionVisible: " << m_sectionVisible;
+}
+
+void ApplicationSettings::notifyWordsetChanged()
+{
+    if(!m_wordset.isEmpty() &&
+       DownloadManager::getInstance()->haveLocalResource(m_wordset) &&
+       !DownloadManager::getInstance()->isDataRegistered("words")) {
+        // words.rcc is there -> register old file first
+        // then try to update in the background
+        DownloadManager::getInstance()->updateResource(m_wordset);
+    }
+
+    updateValueInConfig(GENERAL_GROUP_KEY, WORDSET, m_wordset);
+    qDebug() << "notifyWordset: " << m_wordset;
 }
 
 void ApplicationSettings::notifyDownloadServerUrlChanged()
@@ -302,6 +382,12 @@ void ApplicationSettings::notifyExeCountChanged()
     qDebug() << "exeCount set to: " << m_exeCount;
 }
 
+void ApplicationSettings::notifyLastGCVersionRanChanged()
+{
+    updateValueInConfig(INTERNAL_GROUP_KEY, LAST_GC_VERSION_RAN, m_lastGCVersionRan);
+    qDebug() << "lastVersionRan set to: " << m_lastGCVersionRan;
+}
+
 void ApplicationSettings::notifyBarHiddenChanged()
 {
     qDebug() << "is bar hidden: " << m_isBarHidden;
@@ -388,3 +474,4 @@ void ApplicationSettings::init()
 	qmlRegisterSingletonType<ApplicationSettings>("GCompris", 1, 0,
 												  "ApplicationSettings", systeminfoProvider);
 }
+
